/*
 * Copyright (c) 2005-2010 Thierry FOURNIER
 * $Id: alertes.c 690 2008-03-31 18:36:43Z  $
 *
 */

char *alert_type[] = {
	"ip_change",
	"unknow_address",
	"black_listed",
	"new",  
	"unauthrq",
	"rqabus",
	"mac_error",
	"flood",
	"new_mac",
	"mac_change",
	"mac_expire",
};
